"""Top-level package for confiGOAT."""

__author__ = """Hasan-UL-Banna, Asadullah Al Galib"""
__email__ = 'hasanulbanna04056@gmail.com, asadullahgalib13@gmail.com'
__version__ = '0.1.6'

from .configoat import ConfiGOAT, conf

__all__ = [
    "ConfiGOAT", "conf"
]
